Attribute VB_Name = "Module1"
Option Explicit

Private Type Rect
   Left As Long
   Top As Long
   Right As Long
   Bottom As Long
End Type

Private Type CharRange
  cpMin As Long     ' First character of range (0 for start of doc)
  cpMax As Long     ' Last character of range (-1 for end of doc)
End Type

Private Type FormatRange
  hdc As Long       ' Actual DC to draw on
  hdcTarget As Long ' Target DC for determining text formatting
  rc As Rect        ' Region of the DC to draw to (in twips)
  rcPage As Rect    ' Region of the entire DC (page size) (in twips)
  chrg As CharRange ' Range of text to draw (see above declaration)
End Type

Private Const WM_USER As Long = &H400
Private Const EM_FORMATRANGE As Long = WM_USER + 57
Private Const EM_SETTARGETDEVICE As Long = WM_USER + 72
Private Const PHYSICALOFFSETX As Long = 112
Private Const PHYSICALOFFSETY As Long = 113

Private Declare Function GetDeviceCaps Lib "gdi32" ( _
   ByVal hdc As Long, ByVal nIndex As Long) As Long
Private Declare Function SendMessage Lib "USER32" Alias "SendMessageA" _
   (ByVal hWnd As Long, ByVal msg As Long, ByVal wp As Long, _
   lp As Any) As Long
Private Declare Function CreateDC Lib "gdi32" Alias "CreateDCA" _
   (ByVal lpDriverName As String, ByVal lpDeviceName As String, _
   ByVal lpOutput As Long, ByVal lpInitData As Long) As Long

' Make a RichTextBox control display itself
' using the same parameters as the default printer.
Public Sub WYSIWYG_RTF(RTF As RichTextBox, LeftMarginWidth As Long, RightMarginWidth As Long, TopMarginWidth As Long, BottomMarginWidth As Long, PrintableWidth As Long, PrintableHeight As Long)
Dim LeftOffset As Long
Dim LeftMargin As Long
Dim RightMargin As Long
Dim TopOffset As Long
Dim TopMargin As Long
Dim BottomMargin As Long
Dim PrinterhDC As Long
Dim r As Long

    ' Start a print job to initialize the Printer object.
    Printer.Print " "
    Printer.ScaleMode = vbTwips

    ' Get the left offset to the printable area on the page in twips.
    LeftOffset = GetDeviceCaps(Printer.hdc, PHYSICALOFFSETX)
    LeftOffset = Printer.ScaleX(LeftOffset, vbPixels, vbTwips)

    ' Calculate the left and right margins.
    LeftMargin = LeftMarginWidth - LeftOffset
    RightMargin = (Printer.Width - RightMarginWidth) - LeftOffset
    
    ' Calculate the printable width.
    PrintableWidth = RightMargin - LeftMargin

    ' Get the top offset to the printable area on the page.
    TopOffset = GetDeviceCaps(Printer.hdc, PHYSICALOFFSETY)
    TopOffset = Printer.ScaleX(TopOffset, vbPixels, vbTwips)

    ' Calculate the top and bottom margins.
    TopMargin = TopMarginWidth - TopOffset
    BottomMargin = (Printer.Height - BottomMarginWidth) - TopOffset

    ' Calculate the printable height.
    PrintableHeight = BottomMargin - TopMargin

    ' Create an hDC for the Printer. This DC must
    ' remain for the control to keep up the
    ' WYSIWYG settings.
    PrinterhDC = CreateDC(Printer.DriverName, _
        Printer.DeviceName, 0, 0)

    ' Tell the RichTextBox to base it's display
    ' on the printer at the desired line width.
    r = SendMessage(RTF.hWnd, EM_SETTARGETDEVICE, _
        PrinterhDC, ByVal PrintableWidth)

    ' Cancel the temporary print job used to get
    ' the printer information.
    Printer.KillDoc
End Sub
' Print the contents of a RichTextBox control using the
' indicated margins.
Public Sub PrintRTF(RTF As RichTextBox, LeftMarginWidth As Long, TopMarginHeight, RightMarginWidth, BottomMarginHeight, Optional ByVal header_text As String = "", Optional ByVal footer_text As String = "")
Dim LeftOffset As Long
Dim TopOffset As Long
Dim LeftMargin As Long
Dim TopMargin As Long
Dim RightMargin As Long
Dim BottomMargin As Long
Dim fr As FormatRange
Dim rcDrawTo As Rect
Dim rcPage As Rect
Dim TextLength As Long
Dim NextCharPosition As Long
Dim r As Long
Dim txt As String

    ' Start a print job to get a valid Printer.hDC.
    Printer.Print " "
    Printer.ScaleMode = vbTwips

    ' Get the offset to the page's printable area.
    LeftOffset = Printer.ScaleX(GetDeviceCaps(Printer.hdc, PHYSICALOFFSETX), vbPixels, vbTwips)
    TopOffset = Printer.ScaleY(GetDeviceCaps(Printer.hdc, PHYSICALOFFSETY), vbPixels, vbTwips)

    ' Calculate the left, top, right, and bottom margins.
    LeftMargin = LeftMarginWidth - LeftOffset
    TopMargin = TopMarginHeight - TopOffset
    RightMargin = (Printer.Width - RightMarginWidth) - LeftOffset
    BottomMargin = (Printer.Height - BottomMarginHeight) - TopOffset

    ' Set the printable area rectangle.
    rcPage.Left = 0
    rcPage.Top = 0
    rcPage.Right = Printer.ScaleWidth
    rcPage.Bottom = Printer.ScaleHeight

    ' Set the rectangle in which to print
    ' (relative to the printable area).
    rcDrawTo.Left = LeftMargin
    rcDrawTo.Top = TopMargin
    rcDrawTo.Right = RightMargin
    rcDrawTo.Bottom = BottomMargin

    ' Set up the print instructions.
    fr.hdc = Printer.hdc        ' Use the same DC for measuring and rendering
    fr.hdcTarget = Printer.hdc  ' Point at printer hDC
    fr.rc = rcDrawTo            ' Indicate the area on page to draw to
    fr.rcPage = rcPage          ' Indicate entire size of page
    fr.chrg.cpMin = 0           ' Indicate start of text through
    fr.chrg.cpMax = -1          ' end of the text

    ' Get length of text in RTF
    TextLength = Len(RTF.Text)

    ' Loop printing each page until done
    Do
        ' Print the page by sending EM_FORMATRANGE message
        NextCharPosition = SendMessage(RTF.hWnd, EM_FORMATRANGE, True, fr)
        If NextCharPosition >= TextLength Then Exit Do  'If done then exit
        fr.chrg.cpMin = NextCharPosition ' Starting position for next page

        ' Page number.
        txt = "- " & Format$(Printer.Page) & " -"
        Printer.CurrentX = RightMargin - Printer.TextWidth(txt)
        Printer.CurrentY = Printer.ScaleTop + 1440 * 0.25
        Printer.Print Format$(txt)

        ' Header.
        If Len(header_text) > 0 Then
            Printer.CurrentX = (Printer.ScaleWidth - Printer.TextWidth(header_text)) / 2
            Printer.CurrentY = Printer.ScaleTop + TopMargin / 2
            Printer.Print Format$(header_text)
        End If

        ' Footer.
        If Len(footer_text) > 0 Then
            Printer.CurrentX = (Printer.ScaleWidth - Printer.TextWidth(footer_text)) / 2
            Printer.CurrentY = Printer.ScaleTop + Printer.ScaleHeight + TopMargin / 2
            Printer.Print Format$(footer_text)
        End If
    
        Printer.NewPage                  ' Move on to next page
        Printer.Print " " ' Re-initialize hDC
        fr.hdc = Printer.hdc
        fr.hdcTarget = Printer.hdc
    Loop

    ' End the print job.
    Printer.EndDoc

    ' Allow the RTF to free up memory.
    r = SendMessage(RTF.hWnd, EM_FORMATRANGE, False, ByVal CLng(0))
End Sub
